% Cross Validation algorithm for the DLS algorithm.
% Tomas Puelma 09.2010

function results = DLS_CrossVal(X,Y,genes_agis,pvals,pval_th,nfolds,dw,min_feats,w1_feats_th,delta_w1,dls_min_score,sigma,precW,nFN_out,min_FN_coexp,parallel,conf_th,min_coexp,p_dls_preds,tr_idxs,te_idxs)
% results = DLS_CrossVal(X,Y,nfolds,dw,min_feats,sigma,precW,parallel);
%
% X: data matrix with samples as row vectors.
% Y: column vector with classification (1 or -1) of each X sample.
% nfold: number of folds to use in the cross validation.
%
% results: struct with the results of the cross-validation.
    
    n = size(X,1);
    
    if ~exist('te_idxs','var')
        % make cv folds
        [tr_idxs,te_idxs] = make_crossval_sets(Y,nfolds);
    end
    
    % initialize results variables
    results.nfolds = nfolds;
    results.tr_idxs = tr_idxs;
    results.te_idxs = te_idxs;
    results.Y = Y;
    
    results.params.pval_th = pval_th;
    results.params.dw = dw;
    results.params.min_feats = min_feats;
    results.params.w1_feats_th = w1_feats_th;
    results.params.delta_w1 = delta_w1;
    results.params.dls_min_score = dls_min_score;
    results.params.sigma = sigma;
    results.params.precW = precW;
    results.params.nFN_out = nFN_out;
    results.params.min_FN_coexp = min_FN_coexp;
    results.params.conf_th = conf_th;
    results.params.min_coexp = min_coexp;
    results.params.p_dls_preds = p_dls_preds;
    
    results.nsamples = n;
    results.ntr1 = numel(find(Y(tr_idxs{1}) == 1));
    results.ntr0 = numel(find(Y(tr_idxs{1}) == -1));
    results.nte1 = numel(find(Y(te_idxs{1}) == 1));
    results.nte0 = numel(find(Y(te_idxs{1}) == -1));
    
    results.training.tr_results = cell(1,nfolds);
    results.training.fold_precision = zeros(1,nfolds);
    results.training.fold_recall = zeros(1,nfolds);
    results.training.fold_F = zeros(1,nfolds);
    
    results.test.cl_results = cell(1,nfolds);
    results.test.fold_precision = zeros(1,nfolds);
    results.test.fold_recall = zeros(1,nfolds);
    results.test.fold_F = zeros(1,nfolds);
    
    results.preds.fold = zeros(size(Y));
    results.preds.genes_agis = genes_agis;
    
    results.preds.max_confs = zeros(size(Y));
    results.preds.max_confs_dls_idxs = zeros(size(Y));
    results.preds.max_confs_dls_agis = cell(size(Y));
    results.preds.max_confs_dls_coexps = zeros(size(Y));
    results.preds.max_confs_dls_scores = zeros(size(Y));
    results.preds.max_confs_dls_n_feats = zeros(size(Y));
    
    results.preds.max_scores = zeros(size(Y));
    results.preds.max_scores_dls_idxs = zeros(size(Y));
    results.preds.max_scores_dls_agis = cell(size(Y));
    
    results.preds.max_coexps = zeros(size(Y));
    results.preds.max_coexps_dls_idxs = zeros(size(Y));
    results.preds.max_coexps_dls_agis = cell(size(Y));
    
    results.preds.confs_sum = zeros(size(Y));
    results.preds.scores_sum = zeros(size(Y));
    results.preds.coexps_sum = zeros(size(Y));
    
    results.preds.n_dls_preds = zeros(size(Y));
    
%     results.preds.dls_n_feats = zeros(size(Y));
%     results.preds.dls_scores = zeros(size(Y));
    results.preds.potential_FNs_mask = false(size(Y));
    
    results.H = zeros(size(Y)) - 1;
    results.acc = 0;
    results.precision1 = 0;
    results.recall1 = 0;
    results.precision0 = 0;
    results.recall0 = 0;
    results.F = 0;
    
    nTP = zeros(1,nfolds);
    nFP = zeros(1,nfolds);
    nTN = zeros(1,nfolds);
    nFN = zeros(1,nfolds);
    
    opened = false;
    if parallel && matlabpool('size') == 0
        matlabpool('open');
        opened = true;
    end
    
    if ~parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    % train and test in each fold
    for i = 1:nfolds
        
        t = tic;
        
        fprintf('\n\nCross validation Fold N%d/%d\n',i,nfolds)
        
        % train classifier
        X_tr = X(tr_idxs{i},:);
        Y_tr = Y(tr_idxs{i});
        pvals_tr = pvals(tr_idxs{i},:);
        genes_agis_tr = genes_agis(tr_idxs{i});
        
        results.training.tr_results{i} = DLSM_train(X_tr,Y_tr,pvals_tr,pval_th,genes_agis_tr,dw,min_coexp,min_feats,w1_feats_th,delta_w1,dls_min_score,nFN_out,min_FN_coexp,parallel);
        fprintf('Num valid DLSs: %d',results.training.tr_results{i}.n_valid_dls);
        avg_dls_feats = mean(results.training.tr_results{i}.dls_n_feats);
        avg_dls_avail_feats = mean(results.training.tr_results{i}.dls_n_avail_feats);
        fprintf('\nAvg num selected feats: %.1f/%.1f',avg_dls_feats,avg_dls_avail_feats);
        
        if p_dls_preds == 0
            min_dls_preds = 1;
        else
            min_dls_preds = ceil(p_dls_preds*results.training.tr_results{i}.n_valid_dls);
        end
        fprintf('\nmin dls preds: %d',min_dls_preds);
        
        results.training.cl_results{i} = DLSM_classify(X_tr,genes_agis_tr,results.training.tr_results{i},sigma,parallel,conf_th,min_coexp,min_dls_preds,true);
        
        % evaluate training predictions
        [results.training.fold_F(i),results.training.fold_precision(i),results.training.fold_recall(i)] = F_eval(results.training.cl_results{i}.preds.H,Y_tr,precW);
        
        % save potential FNs
        results.preds.potential_FNs_mask(tr_idxs{i}(results.training.tr_results{i}.potential_FNs_idxs)) = true;
        
        % use classifier
        X_te = X(te_idxs{i},:);
        Y_te = Y(te_idxs{i});
        genes_agis_te = genes_agis(te_idxs{i});
        results.test.cl_results{i} = DLSM_classify(X_te,genes_agis_te,results.training.tr_results{i},sigma,parallel,conf_th,min_coexp,min_dls_preds,false);
        
        H_te = results.test.cl_results{i}.preds.H;
        results.H(te_idxs{i}) = H_te;
        results.preds.fold(te_idxs{i}) = ones(size(te_idxs{i}))*i;
%         results.preds.dls_n_feats(te_idxs{i}) = results.test.cl_results{i}.preds.dls_n_feats;
%         results.preds.dls_scores(te_idxs{i}) = results.test.cl_results{i}.preds.dls_scores;
        
        results.preds.max_confs(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs;
        results.preds.max_scores(te_idxs{i}) = results.test.cl_results{i}.preds.max_scores;
        results.preds.max_coexps(te_idxs{i}) = results.test.cl_results{i}.preds.max_coexps;
        
        results.preds.confs_sum(te_idxs{i}) = results.test.cl_results{i}.preds.confs_sum;
        results.preds.scores_sum(te_idxs{i}) = results.test.cl_results{i}.preds.scores_sum;
        results.preds.coexps_sum(te_idxs{i}) = results.test.cl_results{i}.preds.coexps_sum;
        
        results.preds.n_dls_preds(te_idxs{i}) = results.test.cl_results{i}.preds.n_dls_preds;
        
        results.preds.max_confs_dls_idxs(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs_dls_idxs;
        results.preds.max_confs_dls_agis(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs_dls_agis;
        results.preds.max_confs_dls_coexps(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs_dls_coexps;
        results.preds.max_confs_dls_scores(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs_dls_scores;
        results.preds.max_confs_dls_n_feats(te_idxs{i}) = results.test.cl_results{i}.preds.max_confs_dls_n_feats;
        
        results.preds.max_scores_dls_idxs(te_idxs{i}) = results.test.cl_results{i}.preds.max_scores_dls_idxs;
        results.preds.max_scores_dls_agis(te_idxs{i}) = results.test.cl_results{i}.preds.max_scores_dls_agis;
        
        results.preds.max_coexps_dls_idxs(te_idxs{i}) = results.test.cl_results{i}.preds.max_coexps_dls_idxs;
        results.preds.max_coexps_dls_agis(te_idxs{i}) = results.test.cl_results{i}.preds.max_coexps_dls_agis;
        
        % evaluate results
        [results.test.fold_F(i),results.test.fold_precision(i),results.test.fold_recall(i)] = F_eval(H_te,Y_te,precW);
        
        nTP(i) = numel(find(H_te.*Y_te == 1 & H_te == 1));
        nFP(i) = numel(find(H_te.*Y_te == -1 & H_te == 1));
        nTN(i) = numel(find(H_te.*Y_te == 1 & H_te == -1));
        nFN(i) = numel(find(H_te.*Y_te == -1 & H_te == -1));
        
        % display current performance
        fprintf('\nTraining Precision: %.2f',results.training.fold_precision(i));
        fprintf('\nTraining Recall: %.2f',results.training.fold_recall(i));
        fprintf('\nTraining F: %.2f\n',results.training.fold_F(i));
        fprintf('\nTest Precision: %.2f',results.test.fold_precision(i));
        fprintf('\nTest Recall: %.2f',results.test.fold_recall(i));
        fprintf('\nTest F: %.2f\n',results.test.fold_F(i));
        
        toc(t)
        
    end
    
    if opened && parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    % get overall training and test results
    results.training.mean_precision = mean(results.training.fold_precision);
    results.training.mean_recall = mean(results.training.fold_recall);
    results.training.mean_F = mean(results.training.fold_F);
    
    results.training.std_precision = std(results.training.fold_precision);
    results.training.std_recall = std(results.training.fold_recall);
    results.training.std_F = std(results.training.fold_F);
    
    results.test.mean_precision = mean(results.test.fold_precision);
    results.test.mean_recall = mean(results.test.fold_recall);
    results.test.mean_F = mean(results.test.fold_F);
    
    results.test.std_precision = std(results.test.fold_precision);
    results.test.std_recall = std(results.test.fold_recall);
    results.test.std_F = std(results.test.fold_F);
    
    % get total results
    nTP = sum(nTP);
    nFP = sum(nFP);
    nTN = sum(nTN);
    nFN = sum(nFN);
    
    results.acc = (nTP + nTN)/(nTP + nTN + nFP + nFN);
    results.precision1 = nTP/(nTP + nFP);
    results.recall1 = nTP/(nTP + nFN);
    results.precision0 = nTN/(nTN + nFN);
    results.recall0 = nTN/(nTN + nFP);
    
    results.nTP = nTP;
    results.nFP = nFP;
    results.nTN = nTN;
    results.nFN = nFN;
    
    results.F = (1+precW^2)/((1/results.recall1) + (precW^2/results.precision1));
    
    [th,H] = get_DecisionStump_maxF(results.preds.max_confs,Y,precW);
    
    [F,prec,rec,nTP,nFP] = F_eval(H,Y,precW);
    
    results.max_F = F;
    results.max_F_H = H;
    results.max_F_precision = prec;
    results.max_F_recall = rec;
    results.max_F_nTP = nTP;
    results.max_F_nFP = nFP;
    results.max_F_conf_th = th;
    
end