function results = FND(ExpData,pvals,pval_th,class1_agis,class0_agis,conf_th,p_dls_preds,max_iters,max_p_fns_per_iter,dw,fn_dls_dw,min_feats,w1_feats_th_prctile,min_w1,dls_min_score,sigma,min_coexp,fn_pFN,pFN,min_FN_coexp,score_pos_prctile,add_FNs,parallel)
%
% INPUTS:
% class1_agis: CellArray, contiene los agis de los genes de la clase 1.
% class0_agis: CellArray, contiene los agis de los genes de la clase 0.
% fn_pFN: double [0-1], porcentaje de genes de la clase 0 que se tomarn como
% potenciales falsos negativos.
% fn_conf_th: double [0-1], threshold de confianza para considerar un gene
% como falso negativo.
% max_iters: integer [1-...], numero maximo de iteraciones para buscar FNs.
% dw: double [0-...], peso que se le da a la clase0 en el entrenamiento.
% min_feats: integer [3-...], numero minimo de caracteristicas que pueden
% ser seleccionadas en el entrenamiento.
% dls_min_score: double, score minimo para condirar un DLS valido.
% sigma: double > 0, desviacion estandar utilizada para determinar las
% confianzas en la clasificacion.
% parallel: boolean [true,false], determina si se utilizara procesamiento
% en paralelo (multiprocessor).

    results.params.pval_th = pval_th;
    results.params.add_FNs = add_FNs;
    results.params.fn_pFN = fn_pFN;
    results.params.min_FN_coexp = min_FN_coexp;
    results.params.conf_th = conf_th;
    results.params.p_dls_preds = p_dls_preds;
    results.params.max_iters = max_iters;
    results.params.max_p_fns_per_iter = max_p_fns_per_iter;
    results.params.pFN = pFN;
    results.params.pval_th = pval_th;
    results.params.dw = dw;
    results.params.min_feats = min_feats;
    results.params.w1_feats_th_prctile = w1_feats_th_prctile;
    results.params.min_w1 = min_w1;
    results.params.dls_min_score = dls_min_score;
    results.params.sigma = sigma;
    results.params.min_coexp = min_coexp;
    
    results.params.fn_dls_dw = fn_dls_dw;
    results.params.score_pos_prctile = score_pos_prctile;
    results.params.fn_min_w1 = min_w1;
    
    fn_dls_dw = results.params.fn_dls_dw;
    fn_min_w1 = results.params.fn_min_w1;

    results.valid_FNs_agis = {};
    results.valid_FNs_scores_as_dls = [];
    results.min_score_pos = [];
    
    t = tic;
    
    opened = false;
    if parallel && matlabpool('size') == 0
        matlabpool('open');
        opened = true;
    end
    
    if ~parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    for i = 1:max_iters
        
        t2 = tic;
        fprintf('\nIter %d/%d\n',i,max_iters)
        
        % make training data
        [X,Y,genes_agis,pvals_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis,pvals);
        
        % train
        fn_nFN = ceil(numel(find(Y == -1))*fn_pFN/100);
        tr_results = DLSM_train(X,Y,pvals_X,pval_th,genes_agis,dw,min_coexp,min_feats,w1_feats_th_prctile,min_w1,dls_min_score,fn_nFN,min_FN_coexp,parallel);
        valid_dls_idxs = tr_results.dls_idxs(tr_results.valid_dls_mask);

        fns_agis = genes_agis(tr_results.potential_FNs_idxs);
        fprintf('Num potential FNs: %d\n',numel(fns_agis));
        
        % classify potential FNs
        [X_FNs,~,fns_agis] = makeXY(ExpData.LogRatio,{},fns_agis,ExpData.genes_agis);
        
        min_dls_preds = ceil(p_dls_preds*tr_results.n_valid_dls);
        cl_results = DLSM_classify(X_FNs,fns_agis,tr_results,sigma,parallel,conf_th,min_coexp,min_dls_preds,false);
        
        % select as FNs those with confidence greater than conf_th
        FNs_mask = cl_results.preds.H == 1;
        fns_agis = fns_agis(FNs_mask);
        fprintf('Num potential FNs: %d\n',numel(fns_agis));
        
        if numel(fns_agis) == 0
            toc(t2)
            return;
        end
        
        % select subset of fn_nFN best scored FNs:
        [sorted_scores,sorted_idxs] = sort(cl_results.preds.score_sum_preds(FNs_mask),'descend');
        n_pot_fns = min(fn_nFN*2,numel(fns_agis));
        fns_agis = fns_agis(sorted_idxs(1:n_pot_fns));
        
        % select as FNs those that are valid signatures for the pos class
        n0 = numel(find(Y == -1));
        n_fns = numel(fns_agis);
        nFN = ceil(n0*pFN/100);
        [~,fns_idxs] = ismember(fns_agis,genes_agis);
        valid_fns = false(n_fns,1);
        
        tr_results = DLSM_train(X,Y,pvals_X,pval_th,genes_agis,fn_dls_dw,min_coexp,min_feats,w1_feats_th_prctile,fn_min_w1,dls_min_score,nFN,min_FN_coexp,parallel,valid_dls_idxs);
        min_score_pos = prctile(tr_results.dls_scores_pos(tr_results.valid_dls_mask),results.params.score_pos_prctile);
        results.min_score_pos(i) = min_score_pos;
        fprintf('Min positive score: %.2f\n',min_score_pos);
        
        FNs_scores = zeros(n_fns,1);
        FNs_pos_scores = zeros(n_fns,1);
        
        parfor j = 1:n_fns
            Y_temp = Y;
            Y_temp(fns_idxs(j)) = 1;
            dls = dls_train(X,Y_temp,pvals_X,pval_th,fns_idxs(j),fn_dls_dw,min_coexp,min_feats,w1_feats_th_prctile,fn_min_w1,dls_min_score,nFN,min_FN_coexp);
            
            if dls.is_valid
                if dls.scores.score > 0 && dls.scores.score_pos > min_score_pos
                    valid_fns(j) = true;
                end
                FNs_scores(j) = dls.scores.score;
                FNs_pos_scores(j) = dls.scores.score_pos;
            end
            
        end
        
        new_fns = fns_agis(valid_fns);
        valid_FNs_scores = FNs_scores(valid_fns);
        valid_FNs_pos_scores = FNs_pos_scores(valid_fns);
        [~,sorted_idxs] = sort(valid_FNs_scores,'descend');
        max_fns = ceil(max_p_fns_per_iter*n0);
        n_fns = min(max_fns,numel(new_fns));
        fprintf('max_iter_fns: %d\n',max_fns);
        valid_fn_idxs = sorted_idxs(1:n_fns);
        new_fns = new_fns(valid_fn_idxs);
        valid_fns = valid_fns(valid_fn_idxs);
        
        fprintf('Num valid FNs: %d\n',numel(new_fns));
        fprintf('Valid FNs mean score: %.2f\n',mean(valid_FNs_scores(valid_fn_idxs)));
        fprintf('Valid FNs mean positive score: %.2f\n',mean(valid_FNs_pos_scores(valid_fn_idxs)));
        
        if numel(new_fns) == 0
            return;
        end
        
        results.valid_FNs_agis = [results.valid_FNs_agis; new_fns];
        results.valid_FNs_scores_as_dls = [results.valid_FNs_scores_as_dls; FNs_scores(valid_fns)];
        class0_agis = setdiff(class0_agis,new_fns);
        
        if add_FNs
            class1_agis = union(class1_agis,new_fns);
            fprintf('Adding FNs to class1... \nnclass1 = %d\nnclass0 = %d\n',numel(class1_agis),numel(class0_agis))
        end
            
        toc(t2)

    end
    
    fprintf('\nTotal number of valid FNs: %d\n', numel(results.valid_FNs_agis))
    
    if add_FNs
        fprintf('Final training set:\nnclass1 = %d\nnclass0 = %d\n',numel(class1_agis),numel(class0_agis))
    end
    
    [~,fn_idxs] = ismember(results.valid_FNs_agis,genes_agis);
    results.FNs_tr_results = DLSM_train(X,Y,pvals_X,pval_th,genes_agis,fn_dls_dw,min_coexp,min_feats,w1_feats_th_prctile,fn_min_w1,dls_min_score,nFN,min_FN_coexp,parallel,fn_idxs);
    
    if opened && parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    toc(t);
    disp('Ready!')

end