% Decision-Stump training for a given feature with sample's weights as 
% input.
% Tomas Puelma 05.2010
%
% [th,H,H_real,conf_gt,conf_lt,F] = get_DecisionStump_maxF(x,Y,precW);
%
% Inputs:
% x: column vector with values of a given feature for each sample.
% Y: column vector with the class (1, -1) of each x sample.
% precW: precision weight (respect to recall)
%
% Outputs:
% th: threshold selected to make the partition.
% F: F-meassure of the partition.

function [th,H,H_real,conf_gt,conf_lt,F] = get_DecisionStump_maxF(x,Y,precW)

    % Get all possible thresholds
    ths = unique(x);
    if numel(ths) > 1
        ths(1) = [];
    end
    
    % Initialize variables
    nt = numel(ths);
    F = zeros(nt,1);
    
    % get class statistics    
    c1_mask = Y == 1;
    
    % Test each threshold
    for i = 1:nt
        
        % Get partition
        gt_mask = x >= ths(i);
        
        % assume gt -> class 1 and get F
        c0_gt_mask = gt_mask & ~c1_mask;
        c1_gt_mask = gt_mask & c1_mask;
        
        nTP = numel(find(c1_gt_mask));
        nFP = numel(find(c0_gt_mask));
        n1 = numel(find(c1_mask));
        
        prec = nTP/(nTP + nFP);
        recall = nTP/n1;
        
        F(i) = (1+precW^2)/((1/recall) + (precW^2/prec));
        
    end
    
    % Get threshold with max F
    [F,th_idx] = max(F);
    if th_idx == 1
        th = ths(1);
    else
        th = (ths(th_idx) + ths(th_idx-1))/2;
    end
    
    h1_mask = x >= th;
    conf_gt = numel(find(h1_mask & c1_mask))/numel(find(h1_mask));
    conf_lt = numel(find(~h1_mask & c1_mask))/numel(find(~h1_mask));
    
    H = zeros(size(Y));
    H(h1_mask) = 1;
    H(~h1_mask) = -1;
    
    H_real = zeros(size(Y));
    H_real(h1_mask) = conf_gt;
    H_real(~h1_mask) = conf_lt;

end