function cv_results = libSVM_CV(X,Y,nfolds,tr_params,te_params,conf_th,precW,tr_idxs,te_idxs)

% training params:
% -s svm_type : set type of SVM (default 0)
% 	0 -- C-SVC
% 	1 -- nu-SVC
% 	2 -- one-class SVM
% 	3 -- epsilon-SVR
% 	4 -- nu-SVR
% -t kernel_type : set type of kernel function (default 2)
% 	0 -- linear: u'*v
% 	1 -- polynomial: (gamma*u'*v + coef0)^degree
% 	2 -- radial basis function: exp(-gamma*|u-v|^2)
% 	3 -- sigmoid: tanh(gamma*u'*v + coef0)
% 	4 -- precomputed kernel (kernel values in training_set_file)
% -d degree : set degree in kernel function (default 3)
% -g gamma : set gamma in kernel function (default 1/num_features)
% -r coef0 : set coef0 in kernel function (default 0)
% -c cost : set the parameter C of C-SVC, epsilon-SVR, and nu-SVR (default 1)
% -n nu : set the parameter nu of nu-SVC, one-class SVM, and nu-SVR (default 0.5)
% -p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)
% -m cachesize : set cache memory size in MB (default 100)
% -e epsilon : set tolerance of termination criterion (default 0.001)
% -h shrinking : whether to use the shrinking heuristics, 0 or 1 (default 1)
% -b probability_estimates : whether to train a SVC or SVR model for probability estimates, 0 or 1 (default 0)
% -wi weight : set the parameter C of class i to weight*C, for C-SVC (default 1)
% -v n: n-fold cross validation mode
% -q : quiet mode (no outputs)
% 
% 
% The k in the -g option means the number of attributes in the input data.
% 
% option -v randomly splits the data into n parts and calculates cross
% validation accuracy/mean squared error on them.

% test params:
% -b probability_estimates: whether to predict probability estimates, 0 or 1 (default 0); for one-class SVM only 0 is supported
% 
% model_file is the model file generated by svm-train.
% test_file is the test data you want to predict.
% svm-predict will produce output in the output_file.

    if ~exist('te_idxs','var')
        [tr_idxs,te_idxs] = make_crossval_sets(Y,nfolds);
    end
    
    Confs_fold = cell(1,nfolds);
    parfor i = 1:nfolds
        
        %get gamma
        [w1,gamma] = getParamsSVM(X(tr_idxs{i},:), Y(tr_idxs{i}));
        
        tr_params2 = sprintf('%s -g %.4f',tr_params,gamma);
        
        model = svmtrain(Y(tr_idxs{i}), X(tr_idxs{i},:), tr_params2);
        
        [~, ~, Confs01] = svmpredict(Y(te_idxs{i}), X(te_idxs{i},:), model, te_params);
        Confs_fold{i} = Confs01(:,1);
        
    end
    
    Confs = zeros(size(Y));
    for i = 1:nfolds
        Confs(te_idxs{i}) = Confs_fold{i};
    end
    
    H = Confs > conf_th;
    [F,precision,recall,nTP,nFP] = F_eval(H,Y,precW);
    
    cv_results.params.tr_params = tr_params;
    cv_results.params.te_params = te_params;
    cv_results.params.conf_th = conf_th;
    cv_results.params.precW = precW;
    cv_results.H = H;
    cv_results.Confs = Confs;
    cv_results.F = F;
    cv_results.precision = precision;
    cv_results.recall = recall;
    cv_results.nTP = nTP;
    cv_results.nFP = nFP;
    
    [~,recalls,precisions,ths] = rocRP(Confs,Y,false);
    F = (1+precW^2)./((1./recalls) + (precW^2./precisions));
        
    [cv_results.max_F,idx] = max(F);
    cv_results.max_F_conf_th = ths(idx);
    cv_results.max_F_precision = precisions(idx);
    cv_results.max_F_recall = recalls(idx);  
    cv_results.max_F_H = Confs >= cv_results.max_F_conf_th;

end