% script to run DLS in a custom biological process of interest

%% addpaths
addpath ../Functions/DLS_Core/
addpath ../Functions/DCN/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Functions/PrintsResults/
addpath ../Data/

%% specify parameters

test_name = 'response_to_nitrate';

directory = '../Results/';
if ~exist(directory,'dir')
    mkdir(directory);
end

% general params
params.fns_discovery = false;   % do you want to run FND before running DLS?
params.do_cv = true;           % do you want to run a cross-validation over the training set to estimate the accuracy of predictions?
params.plot_dcn = true;         % do you want for DLS to construct a discriminative coexpression network with the predictions?
params.parallel = true;         % do you want for DLS to use all the cores in your computer?

% training params
params_tr.sign_min_prec = 0.75;  % Discriminative stringency factor (0-1)
params_tr.dw = params_tr.sign_min_prec/(1-params_tr.sign_min_prec);     % w0: automatically calculated
params_tr.sign_min_feats = 5;   % signatures minumum number of features
params_tr.sign_min_score = 0;   % minimum ESS score to consider a signature as valid.
params_tr.w1_feats_th_prctile = 30;     % below this percentage of features, the signature's scores are penalized.
params_tr.min_w1 = 0.6;         % maximum penalization
params_tr.pFN = 0.05;           % percentage of false negatives to automatically discard during training (0-100)
params_tr.min_FN_coexp = 0.5;   % minimum coexpression with a negative gene to consider it a potential false negative (0-1)
params_tr.FDR_th = 0.1;
params_tr.min_coexp = 0.6;      % minumum coexpression to accept a prediction (0-1) (optional)

% classification params
params_cl.conf_th = 0.5;        % minimum confidence to accept a prediction (0-1)
params_cl.min_coexp = 0.6;      % minumum coexpression to accept a prediction (0-1) (optional)
params_cl.p_dls_preds = 0;      % minimum percentage of signatures that must predict a gene to consider it a valid prediction (0: default -> one signature)
params_cl.sigma = 0.1;          % bandwith for the kernel density estimation (not recomended to change)

% FNs discovery params
params_fn.max_iters = 10;       % number of FND iterations.
params_fn.addFNs = true;        % do you want the FND to automatically add to the positive class the predicted FNs on each iteration?
params_fn.max_iter_pFNs = 0.01; % maximum percentage of negative genes that can be predicted as FNs on each iteration (0-100).

    % FND step 1 training params
    params_fn_tr.sign_min_prec = 0.9;
    params_fn_tr.dw = params_fn_tr.sign_min_prec/(1-params_fn_tr.sign_min_prec);
    params_fn_tr.sign_min_feats = 5;
    params_fn_tr.sign_min_score = 0;
    params_fn_tr.w1_feats_th_prctile = 40;
    params_fn_tr.min_w1 = 0.6;
    params_fn_tr.pFN = 0.5;
    params_fn_tr.min_FN_coexp = 0.5;
    params_fn_tr.FDR_th = 0.1;
    
    % FND step 2 classification params
    params_fn_cl.conf_th = 0.5;
    params_fn_cl.min_coexp = 0.6;
    params_fn_cl.sigma = 0.1;
    params_fn_cl.p_dls_preds = 0;
    
    % FN step 3 signatures params
    params_fn_sign.score_pos_prctile = 50;
    params_fn_sign.pFN = 0.5;
    
% cross-validation params
params_cv.use_all_c1_folds = true;  % do you want for the cross-validation to use as many folds as the number of positive genes? (can take a long time)
params_cv.cv_nfolds = 10;           % if not, then how many folds do you want to run?
params_cv.precW = 2;                % Beta parameter to use in the F-score evaluation metric


% training set construction params
params_tr_set.max_valid_p1 = 0.05;           % maxumim percentage of positive genes that can be annotated in a go-term to be considered a negative go-term
params_tr_set.min_valid_terms_size = 30;     % minimum number of genes that must be annotated in a go-term to be considered a negative go-term
params_tr_set.min_valid_terms = 1;           % minimum number of negative go-terms where a gene must be directly annotated to be considered negative
params_tr_set.max_invalid_terms = 0;         % maximum number of non-negative go-terms where a gene can be annotated to be considered negative

% specify the go-terms that define your biological process of interest
params_tr_set.go_terms = [
    10167 % Response to Nitrate
    42128 % Nitrate Assimilation
    15706 % Nitrate Transport
    ];

% change training set with custom genes
params_tr_set.add_to_class1 = {
%     'AT5G40850' % UPM1
%     'AT5G65010' % ASN2
};
params_tr_set.add_to_class0 = {
};
params_tr_set.add_to_unlabeled = {
%     'AT5G13110' % G6PD2
%     'AT1G24280' % G6PD3
%     'AT5G14760' % AO
%     'AT2G26980' % CIPK3
};

%% initialize

% load expression data
load ExpData_automated.mat

% load annotation matrices
load Arabidopsis_annot_matrix_P_All(2010.09.07).mat
annot_matrix_all = annot_matrix;

load Arabidopsis_annot_matrix_P_nonIEA(2010.09.07).mat
annot_matrix_nonIEA = annot_matrix;

load Arabidopsis_direct_annot_matrix_P_All(2010.09.07).mat
direct_annot_matrix_all = direct_annot_matrix;

clear annot_matrix direct_annot_matrix

% load gene aliases for the results outputs
load GeneAliases.mat

%% construct training set

go_terms = num2goid(params_tr_set.go_terms);

classes_data = get_goterm_sets_direct(go_terms,annot_matrix_all,direct_annot_matrix_all,annot_matrix_nonIEA,params_tr_set.max_invalid_terms,params_tr_set.min_valid_terms,params_tr_set.min_valid_terms_size,params_tr_set.max_valid_p1);

class1_agis = classes_data.class1_agis;
class0_agis = classes_data.class0_agis;
unlabeled_agis = classes_data.unlabeled_agis;

disp('Selected training set based in GO annotations:')
fprintf('Positive genes: %d\n',numel(class1_agis))
fprintf('Negative genes: %d\n',numel(class0_agis))
fprintf('Unlabeled genes: %d\n',numel(unlabeled_agis))

%% change training set

add_to_class1 = params_tr_set.add_to_class1;
add_to_class0 = params_tr_set.add_to_class0;
add_to_unlabeled = params_tr_set.add_to_unlabeled;

if ~isempty([add_to_class1; add_to_class0; add_to_unlabeled])
    if any(~ismember([add_to_class1; add_to_class0; add_to_unlabeled],ExpData.genes_agis))
        error('invalid agi')
    end

    class0_agis = setdiff(class0_agis,union(add_to_class1,add_to_unlabeled));
    class1_agis = setdiff(class1_agis,union(add_to_class0,add_to_unlabeled));
    unlabeled_agis = setdiff(unlabeled_agis,union(add_to_class1,add_to_class0));

    class1_agis = union(class1_agis,add_to_class1);
    class0_agis = union(class0_agis,add_to_class0);
    unlabeled_agis = union(unlabeled_agis,add_to_unlabeled);

    classes_data.class1_agis = class1_agis;
    classes_data.class0_agis = class0_agis;
    classes_data.unlabeled_agis = unlabeled_agis;

    disp('Modified training set:')
    fprintf('Positive genes: %d\n',numel(class1_agis))
    fprintf('Negative genes: %d\n',numel(class0_agis))
    fprintf('Unlabeled genes: %d\n',numel(unlabeled_agis))
end

%% FNs discovery

if params.fns_discovery
    
    disp('Discovering potential false negatives...')

    FNs_results = FND(ExpData,ExpData.FDR,params_fn_tr.FDR_th,class1_agis,class0_agis,params_fn_cl.conf_th,params_fn_cl.p_dls_preds,params_fn.max_iters,params_fn.max_iter_pFNs,params_fn_tr.dw,params_fn_tr.dw,params_fn_tr.sign_min_feats,params_fn_tr.w1_feats_th_prctile,params_fn_tr.min_w1,params_fn_tr.sign_min_score,params_fn_cl.sigma,params_fn_cl.min_coexp,params_fn_sign.pFN,params_fn_tr.pFN,params_fn_tr.min_FN_coexp,params_fn_sign.score_pos_prctile,params_fn.addFNs,params.parallel);

    % extract FNs results
    if ~isempty(FNs_results.valid_FNs_agis)
        fns_agis = FNs_results.valid_FNs_agis;
        class0_agis = setdiff(class0_agis,fns_agis);
        if addFNs
            class1_agis = union(class1_agis,fns_agis);
        else
            unlabeled_agis = union(unlabeled_agis,fns_agis);
        end
        
        disp('FN discovery updated training set:')
        fprintf('Positive genes: %d\n',numel(class1_agis))
        fprintf('Negative genes: %d\n',numel(class0_agis))
        fprintf('Unlabeled genes: %d\n',numel(unlabeled_agis))
        
        load GeneAliases.mat
        save_name = strcat(directory,test_name,'_Predicted_FNs_Signatures.txt');
        print_DLS_signatures(FNs_results.FNs_tr_results,save_name,GenesAliases,params_cl.conf_th)
        fprintf('\nFND results details saved in:\n%s',save_name);
    else
        FNs_results = nan;
    end

    disp('Ready');
else
    FNs_results = nan;
end

%% train DLS

tic
disp('Training...')

% get data
if ~exist('ExpData','var')
    load ExpData_automated.mat
end
[X,Y,genes_agis_X,pvals_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis,ExpData.FDR);

nFN = round(numel(class0_agis)*params_tr.pFN/100);
TR_results = DLSM_train(X,Y,pvals_X,params_tr.FDR_th,genes_agis_X,params_tr.dw,params_tr.min_coexp,params_tr.sign_min_feats,params_tr.w1_feats_th_prctile,params_tr.min_w1,params_tr.sign_min_score,nFN,params_tr.min_FN_coexp,params.parallel);
       
save_name = strcat(directory,test_name,'_TR_Signatures.txt');
print_DLS_signatures(TR_results,save_name,GenesAliases)
fprintf('\nSigantures details saved in:\n%s',save_name);

disp('Training ready!')
toc

%% make new predictions

tic
disp('Classifying...')

if ~exist('ExpData','var')
    load ExpData_automated.mat
end

% classify training set
min_dls_preds = ceil(params_cl.p_dls_preds*TR_results.n_valid_dls);
TR_cl_results = DLSM_classify(X,genes_agis_X,TR_results,params_cl.sigma,params.parallel,params_cl.conf_th,params_cl.min_coexp,min_dls_preds,true);

[temp,unlabeled_idxs] = ismember(unlabeled_agis,ExpData.genes_agis);
X_new = ExpData.LogRatio(unlabeled_idxs,:);
genes_agis_new = ExpData.genes_agis(unlabeled_idxs);

% classify new genes
new_results = DLSM_classify(X_new,genes_agis_new,TR_results,params_cl.sigma,params.parallel,params_cl.conf_th,params_cl.min_coexp,min_dls_preds,false);
        
% print results summary
print_DLS_preds(TR_cl_results,params_cl.conf_th,directory,strcat(test_name,'_TR'),GenesAliases,Y);
print_DLS_preds(new_results,params_cl.conf_th,directory,strcat(test_name,'_NEW_results'),GenesAliases);

disp('Classification ready!')
toc

%% make cross-val

if params.do_cv
    disp('Making cross validation...')

    if ~exist('ExpData','var')
        load ExpData_automated.mat
    end
    [X,Y,genes_agis_X,pvals_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis,ExpData.FDR);

    nFN = round(numel(class0_agis)*params_tr.pFN/100);
    if params_cv.use_all_c1_folds
        params_cv.cv_nfolds = numel(class1_agis);
    end

    t = tic; results_DLS_crossval = DLS_CrossVal(X,Y,genes_agis_X,pvals_X,params_tr.FDR_th,params_cv.cv_nfolds,params_tr.dw,params_tr.sign_min_feats,params_tr.w1_feats_th_prctile,params_tr.min_w1,params_tr.sign_min_score,params_cl.sigma,params_cv.precW,nFN,params_tr.min_FN_coexp,params.parallel,params_cl.conf_th,params_cl.min_coexp,params_cl.p_dls_preds); results_DLS_crossval.time = round(toc(t));

    results_DLS_crossval.auc = rocRP(results_DLS_crossval.preds.max_confs,Y,true);
    rocConfPrec(results_DLS_crossval.preds.max_confs,Y,true);
    rocConfRecall(results_DLS_crossval.preds.max_confs,Y,true);

    % print results summary
    if ~exist('GeneAliases','var')
        load GeneAliases.mat
    end
    print_DLS_preds(results_DLS_crossval,params_cl.conf_th,directory,strcat(test_name,'_CV'),GenesAliases,Y);

    disp('Cross-validation ready!')
else
    results_DLS_crossval = nan;
end

%% Save results in .mat file

disp('Saving results...')
save_name = strcat(directory,test_name,'_mat_results.mat');
save(save_name,'go_terms','classes_data','FNs_results','results_DLS_crossval','TR_results','new_results','TR_cl_results')
fprintf('\nMatlab variables with results saved in:\n%s\n',save_name);

%% plot discriminative coexpression network

if params.plot_dcn
    disp('Constructing network...')
    
    if ~exist('GenesAliases','var')
        load GeneAliases.mat
    end

    % targets_agis = {'AT1G08090'};
    targets_agis = {};
    new_targets_agis = {};

    networkMatrices_tr = plotNetworkMatrix_DLS_target(targets_agis, ExpData.genes_agis, TR_cl_results, TR_results, params_cl.conf_th, true, false, GenesAliases);
    networkMatrices_tr_new = plotNetworkMatrix_DLS_target(new_targets_agis, ExpData.genes_agis, new_results, TR_results, params_cl.conf_th, true, true, GenesAliases, networkMatrices_tr);

    save_name = strcat(directory,'Network_TR_NEW_',test_name,'.txt');
    printNetworkResults(networkMatrices_tr_new,ExpData.genes_agis,params_cl.conf_th,save_name,GenesAliases);
    
    fprintf('\nNetwork details saved in:\n%s\n',save_name);
end

fprintf('\n\n\nAll done!!\n');
