% Script to selects a representative set of GO biological processes using
% year 2008 GO annotations and generates the corresponding training sets
% and cross-validation partitions for each of them

%% addpaths

addpath ../Functions/DLS_Core/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data

%% load data

disp('Loading data...')

% load "expert" expression data
load ExpData_expert.mat

% load gene ontology DAG from 2008
GO = my_geneont('FILE','gene_ontology.2008.05.01.obo');

% load Arabidopsis annotations from 2008
load Arabidopsis_annot_matrix_P_nonIEA(2008.05.08).mat
annot_matrix_nonIEA = annot_matrix;

% load Arabidopsis direct annotations from 2008
load Arabidopsis_direct_annot_matrix_P_nonIEA(2008.05.08).mat
direct_annot_matrix_nonIEA = direct_annot_matrix;

%% specify parameters
results_directory = '../Data/';
if ~exist(results_directory,'dir')
    mkdir(results_directory);
end

name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;

cv_nfolds = 10; % number of cross-validation folds to create to each go-term
n_cvreps = 3;

max_invalid_terms = 0;
min_valid_terms = 1;
min_valid_terms_size = 30;
max_valid_p1 = 0.05;

% save parameters in results structure
go_terms.params.name = name;

go_terms.params.year = year;
go_terms.params.min_annots = min_annots;
go_terms.params.max_annots = max_annots;

go_terms.params.cv_nfolds = cv_nfolds;
go_terms.params.n_cvreps = n_cvreps;

go_terms.params.max_invalid_terms = max_invalid_terms;
go_terms.params.min_valid_terms = min_valid_terms;
go_terms.params.min_valid_terms_size = min_valid_terms_size;
go_terms.params.max_valid_p1 = max_valid_p1;

%% Select GO terms to test from GO tree

tic
disp('Getting representative GO terms...')
go_terms = getRepresentativeGoTerms(GO,annot_matrix_nonIEA,min_annots,max_annots);
toc

%% Get genes sets for each class
    
tic
disp('Getting GO terms training sets...')

n_terms = numel(go_terms.id);
class1_agis = cell(n_terms,1);
class0_agis = cell(n_terms,1);
class1_idxs = cell(n_terms,1);
class0_idxs = cell(n_terms,1);
go_terms.nclass1 = zeros(n_terms,1);
go_terms.nclass0 = zeros(n_terms,1);
for i = 1:n_terms

    classes_data = get_goterm_sets_direct(num2goid(go_terms.id(i)),annot_matrix_nonIEA,direct_annot_matrix_nonIEA,annot_matrix_nonIEA,max_invalid_terms,min_valid_terms,min_valid_terms_size,max_valid_p1);

    class1_agis{i} = classes_data.class1_agis;
    class1_idxs{i} = find(ismember(ExpData.genes_agis,class1_agis{i}));
    go_terms.nclass1(i) = numel(class1_agis{i});
    class0_agis{i} = classes_data.class0_agis;
    class0_idxs{i} = find(ismember(ExpData.genes_agis,class0_agis{i}));
    go_terms.nclass0(i) = numel(class0_agis{i});

end
toc

%% Get and save statistics from selected GO terms
% (mean_numDiff = DE_BP^+)

tic
disp('Selecting best subset of GO terms...')

eval_terms_results = evalTermsDiffExp(ExpData.DiffExp_FDR01,ExpData.genes_agis,class1_agis,30);
[~,sorted_idxs] = sort(eval_terms_results.mean_numDiff,'descend');

go_terms.id = go_terms.id(sorted_idxs);
go_terms.level = go_terms.level(sorted_idxs);
go_terms.name = go_terms.name(sorted_idxs);
go_terms.mean_numDiff = eval_terms_results.mean_numDiff(sorted_idxs);
go_terms.class1_agis = class1_agis(sorted_idxs);
go_terms.class0_agis = class0_agis(sorted_idxs);
go_terms.class1_idxs = class1_idxs(sorted_idxs);
go_terms.class0_idxs = class0_idxs(sorted_idxs);
go_terms.nclass1 = go_terms.nclass1(sorted_idxs);
go_terms.nclass0 = go_terms.nclass0(sorted_idxs);
go_terms = rmfield(go_terms, 'num_annots');
clear class1_agis class0_agis class1_idxs class0_idxs
toc

%% Get cross-validation sets

tic
n_terms = numel(go_terms.id);
disp('Getting crossval sets...')

go_terms.tr_idxs = cell(n_terms,n_cvreps);
go_terms.te_idxs = cell(n_terms,n_cvreps);

for i = 1:n_terms

    Y = [ones(go_terms.nclass1(i),1); -ones(go_terms.nclass0(i),1)];
    
    for j = 1:n_cvreps
        [go_terms.tr_idxs{i,j},go_terms.te_idxs{i,j}] = make_crossval_sets(Y,cv_nfolds);
    end

end
toc

%% save go-terms data

tic
save_name = sprintf('%sGO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',results_directory,year,min_annots,max_annots,cv_nfolds,name);
fprintf('Saving results in file: \n%s\n',save_name);
save(save_name,'go_terms')
toc

fprintf('\n\nAll done! ;) Representative GO-terms data saved in variable ''go_terms''.\n')
