% Script to run Coexpression Networks (CN) cross-validation test over the 
% go-terms defined by Paper_test_1_Representative_GO_terms_script.m

%% addpaths

addpath ../Other_methods/CN/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data

%% define parameters
test_name = 'CN'; results.params.test_name = test_name;
directory = '../Results/Expert/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;
n_cvreps = 3;

corr_cutoff = 0.6; results.params.corr_cutoff = corr_cutoff;
pval_th = 0.1; results.params.pval_th = pval_th;
correct_pvals = true; results.params.correct_pvals = correct_pvals;
precW = 2; results.params.precW = precW;

%% load data

tic
disp('Loading data...')

load ExpData_expert.mat

% load go-terms data
load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,name))
results.go_terms = go_terms;
clear go_terms

%% Construct Network

% load pre-computed correlations
if ~exist('CorrVector','var') && ~exist('nborsMatrix','var')
    CorrVector = getCorrVector(ExpData.LogRatio,'cosine');
    clear ExpData
    save(sprintf('%sCorrVector_expert.mat',terms_data_dir),'CorrVector')
    disp('Ready')
end

% load annotations data
load Arabidopsis_annot_matrix_P_All(2008.05.08).mat
genes_agis = annot_matrix.rowHeaders;
ntotal_genes = numel(genes_agis);
clear annot_matrix

if ~exist('nborsMatrix','var')
    disp('Constructing network...')
    nborsMatrix = getNbors_from_vector_cutoff(CorrVector,1:ntotal_genes,ntotal_genes,corr_cutoff);
    clear CorrVector
end

toc

%% Run method
disp('Starting cross-validation analisys...')

n_terms = numel(results.go_terms.id);
results.pval_th = zeros(n_terms,n_cvreps);
results.F = zeros(n_terms,n_cvreps);
results.precision = zeros(n_terms,n_cvreps);
results.recall = zeros(n_terms,n_cvreps);
results.preds.tr_agis = cell(n_terms,n_cvreps);
results.preds.Y = cell(n_terms,n_cvreps);
results.preds.pvals = cell(n_terms,n_cvreps);
results.preds.fps_agis = cell(n_terms,n_cvreps);
results.preds.tps_agis = cell(n_terms,n_cvreps);

t = tic;

for i = 1:n_terms
    
    t2 = tic;
    for j = 1:n_cvreps
        fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\nReplicate: %d\n',results.go_terms.id(i),results.go_terms.name{i},results.go_terms.nclass1(i),results.go_terms.nclass0(i),results.go_terms.mean_numDiff(i),j);

        class0_idxs = results.go_terms.class0_idxs{i};
        class1_idxs = results.go_terms.class1_idxs{i};
        nclass0 = results.go_terms.nclass0(i);
        nclass1 = results.go_terms.nclass1(i);
        tr_idxs = results.go_terms.tr_idxs{i,j};
        te_idxs = results.go_terms.te_idxs{i,j};

        term_results = exprNet_enrich_cv3(nborsMatrix,'nbors',genes_agis,class1_idxs,class0_idxs,corr_cutoff,cv_nfolds,te_idxs);
    
        if j == 3
            print = true;
        else
            print = false;
        end
        
        results.preds.pvals{i,j} = term_results.pvals;
        
        cv_results = getPrintCVResults(results,pval_th,false,correct_pvals,precW,directory,test_name,print,i);
        
        results.pval_th(i,j) = cv_results.pval_ths(i,j);
        results.F(i,j) = cv_results.F(i,j);
        results.precision(i,j) = cv_results.precision(i,j);
        results.recall(i,j) = cv_results.recall(i,j);
        results.preds.tr_agis{i,j} = genes_agis([class1_idxs; class0_idxs]);
        results.preds.Y{i,j} = [ones(nclass1,1); -ones(nclass0,1)];
        results.preds.tps_agis{i,j} = results.preds.tr_agis{i,j}(term_results.pvals < cv_results.pval_ths(i,j) & results.preds.Y{i,j} == 1);
        results.preds.fps_agis{i,j} = results.preds.tr_agis{i,j}(term_results.pvals < cv_results.pval_ths(i,j) & results.preds.Y{i,j} == -1);
        
        fprintf('\nRep%d Done! Cutoff:%.2f, Best pval:%.1e, F:%.2f, Precision:%.2f, Recall:%.2f\n',j,corr_cutoff,results.pval_th(i,j),results.F(i,j),results.precision(i,j),results.recall(i,j))

    end

    toc(t2)
    
end

save_name = sprintf('%sResults_CV_%s.mat',directory,test_name);
save(save_name,'results')
toc(t)

fprintf('\nReady!! Results saved in:\n%s\n',save_name)

